import { useEffect, useState } from 'react';
import { Accordion, AccordionItem, AccordionHead, AccordionBody } from '../components/accordion';
import { TextEditor } from '../components/textEditor';
import {
  InsightResponse,
  updateInsight,
  deleteInsight,
  addInsight,
  InsightRequest,
} from '@graphpolaris/shared/lib/data-access/store/insightSharingSlice';
import { useAppDispatch, useSessionCache } from '@graphpolaris/shared/lib/data-access';
import { Button, Input, LoadingSpinner } from '../components';
import { wsUpdateInsight, wsCreateInsight, wsDeleteInsight } from '@graphpolaris/shared/lib/data-access/broker/wsInsightSharing';
import { addError, addSuccess } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { SerializedEditorState } from 'lexical';

type Props = {
  insight: InsightResponse;
  setActive?: (val: string) => void;
  handleDelete: () => void;
};

export function FormAlert(props: Props) {
  const dispatch = useAppDispatch();
  const session = useSessionCache();
  const [loading, setLoading] = useState(false);
  const [name, setName] = useState(props.insight.name);
  const [description, setDescription] = useState(props.insight.description);
  const [recipients, setRecipients] = useState<string[]>(props.insight.recipients || []);
  const [recipientInput, setRecipientInput] = useState<string>('');
  const [editorState, setEditorState] = useState<SerializedEditorState | null>(null);

  useEffect(() => {
    let isMounted = true;

    if (isMounted) {
      setName(props.insight.name);
      setDescription(props.insight.description);
      setRecipientInput(props.insight.recipients ? props.insight.recipients.join(', ') : '');
      setRecipients(props.insight.recipients || []);
      if (props.insight.template) {
        try {
          const parsedTemplate = JSON.parse(props.insight.template);
          setEditorState(parsedTemplate);
        } catch (e) {
          setEditorState(null);
          console.error('Failed to parse template:', e);
        }
      } else {
        setEditorState(null);
      }
    }

    return () => {
      isMounted = false;
      setEditorState(null);
    };
  }, [props.insight]);

  const handleSave = async (element: SerializedEditorState) => {
    if (!name || name.trim() === '') {
      dispatch(addError('Name is required'));
      return;
    }

    if (!session.currentSaveState) {
      dispatch(addError('No save state selected'));
      return;
    }

    const alertData: InsightRequest = {
      name,
      description,
      recipients,
      template: JSON.stringify(element),
      saveStateId: session.currentSaveState,
      type: 'alert' as const,
      frequency: '',
    };

    setLoading(true);

    if (props.insight.id) {
      wsUpdateInsight(props.insight.id, alertData, (data: any, status: string) => {
        setLoading(false);
        if (status === 'success') {
          dispatch(updateInsight(data));
          dispatch(addSuccess('Alert updated successfully'));
          console.log('Alert updated successfully');
        } else {
          console.error('Failed to update alert:', data);
          dispatch(addError('Failed to update alert.'));
        }
      });
    } else {
      wsCreateInsight(alertData, (data: any, status: string) => {
        setLoading(false);
        if (status === 'success') {
          dispatch(addInsight(data));
          dispatch(addSuccess('Alert created successfully'));
          console.log('Alert created successfully');
          if (props.setActive) props.setActive(data.id);
        } else {
          console.error('Failed to create alert:', data);
          dispatch(addError('Failed to create alert.'));
        }
      });
    }
  };

  const handleDelete = async () => {
    if (!props.insight.id) {
      dispatch(addError('Cannot delete an alert without an ID.'));
      return;
    }

    setLoading(true);
    wsDeleteInsight(props.insight.id, (data: any, status: string) => {
      setLoading(false);
      if (status === 'success') {
        dispatch(deleteInsight({ id: props.insight.id }));
        if (props.setActive) props.setActive('');
        dispatch(addSuccess('Alert deleted successfully'));
        console.log('Alert deleted successfully');
      } else {
        console.error('Failed to delete alert:', data);
        dispatch(addError('Failed to delete alert'));
      }
    });
  };

  if (loading) {
    return <LoadingSpinner />;
  }

  return (
    <div>
      <span className="text-lg text-secondary-600 font-bold mb-4">Alert ID: {props.insight.id}</span>
      <Input label="Name" type="text" value={name} onChange={setName} className="mb-4" />
      <Input label="Description" type="text" value={description} onChange={setDescription} className="mb-4" />
      <Accordion defaultOpenAll={true} className="border-t divide-y">
        <AccordionItem className="pt-2 pb-4">
          <AccordionHead showArrow={false}>
            <span className="font-semibold">Recipient(s)</span>
          </AccordionHead>
          <AccordionBody>
            <div>
              <Input
                type="text"
                value={recipientInput}
                onChange={(value) => {
                  setRecipientInput(String(value));
                  const recipientList = String(value)
                    .split(/[, ]/)
                    .map((r) => r.trim())
                    .filter((r) => r.length > 0);
                  setRecipients(recipientList);
                }}
                placeholder="Enter recipient(s)"
                className="w-full"
              />
            </div>
          </AccordionBody>
        </AccordionItem>
        <AccordionItem className="pt-2 pb-4">
          <AccordionHead showArrow={false}>
            <span className="font-semibold">Alerting text</span>
          </AccordionHead>
          <AccordionBody>
            <TextEditor
              key={`editor-${props.insight.id}`}
              editorState={editorState}
              setEditorState={setEditorState}
              showToolbar={true}
              placeholder="Start typing your alert template..."
              handleSave={handleSave}
            >
              <Button label="Delete" variantType="secondary" variant="outline" onClick={handleDelete} />
            </TextEditor>
          </AccordionBody>
        </AccordionItem>
      </Accordion>
    </div>
  );
}
